from pathlib import Path


def srt_to_txt(filepath: Path) -> str:
    """Extract the text strings from a `.srt` file"""
    # SRT file structure: https://docs.fileformat.com/video/srt/
    assert filepath.suffix.casefold() == ".srt", "Expected `.srt` file"
    with open(filepath, "r", encoding="utf-8") as f:
        data = f.read()
    texts: list[str] = data.split(
        "\n\n"
    )  # Since each line is seperated by 1 blank line
    texts = [
        "\n".join(text.split("\n")[2:]) for text in texts
    ]  # Since each block has a numeric counter followed by timestamp
    return "\n".join(texts)



import tomllib
from functools import cached_property
from pathlib import Path


class Config:
    def __init__(
        self, filename: str = "config.toml", filepath: Path | None = None
    ) -> None:
        self.__filename = filename
        self.__filepath: Path | None = None
        if filepath is not None:
            if filepath.is_dir():
                self.__filepath = filepath / filename
            else:
                self.__filepath = filepath.name
        else:
            self.__filepath = Path(filename)

        assert self.__filepath.exists()

    def __str__(self) -> str:
        return f"Config(filename={self.__filename}, filepath={self.__filepath})"

    @property
    def filepath(self) -> Path:
        return self.__filepath

    @property
    def value(self) -> dict:
        """Get config value"""
        with open(self.filepath, "r") as f:
            return tomllib.loads(f.read())
        return {}

    def get(self, keys: tuple[str, ...], default=None):
        """Get the value from config"""
        data = self.value
        for key in keys[:-1]:
            data = data.get(key, {})
        return data.get(keys[-1], default)



"Transcribe Media Files"

__version__ = "0.0.6"

from rich.console import Console
from rich.panel import Panel

from ak_transcribe import converter
from ak_transcribe.transcriber import Transcriber, TranscribedContent

from rich.console import Console

console = Console()


def test_configs():
    """Tests if the config_file is correctly set up"""
    from ak_transcribe.utils.config_parser import Config

    _config = Config()
    console = Console()

    module_name = __name__
    console.print(f"[bold magenta]Module: {module_name}[/bold magenta]\n")

    def _config_vals_ok(keys: tuple[str, ...], default="NAN") -> bool:
        return _config.get(keys=keys, default=default) != default

    # Define a helper to format the output
    def check_configs(title, vals_to_check):
        with console.status(f"[bold cyan]Checking {title} configs...", spinner="dots"):
            if all([_config_vals_ok(x) for x in vals_to_check]):
                console.print(
                    Panel(f"[green bold]OK", title=title, border_style="green")
                )
            else:
                console.print(
                    Panel(f"[red bold]Not OK", title=title, border_style="red")
                )

    # FFMPEG check
    check_configs("ffmpeg", [("ffmpeg", "cmd"), ("ffmpeg", "transcode_dir")])

    # Whisper-Docker check
    check_configs("Whisper-Docker", [("whisper", "url")])

    # Whisper-CPU check
    check_configs("Wisper-CPU", [("whisper", "model"), ("whisper", "device")])



import os
from pathlib import Path

import pytest

from ak_transcribe.transcriber import *
from ak_transcribe.transcriber import TempMp3


# Define a temporary file for testing purposes
@pytest.fixture
def temp_file(tmpdir):
    # Create a dummy audio file in a temporary directory
    file_path = tmpdir / "test_audio.wav"
    file_path.write(
        "dummy data"
    )  # Writing some dummy data, actual content doesn't matter for these tests
    return Path(file_path)


def test_temp_mp3_conversion(tmpdir, temp_file):
    # Test the conversion of a non-MP3 file to MP3 using ffmpeg
    temp_mp3 = TempMp3(filepath=temp_file)
    with pytest.raises(
        Exception
    ):  # We expect an exception since we're trying to convert a dummy audio file
        with temp_mp3:
            pass  # The conversion should not succeed, and this line should never be reached


def test_temp_mp3_non_conversion(tmpdir, temp_file):
    # Test the scenario where the file is already an MP3 (no conversion should happen)
    temp_mp3 = TempMp3(
        filepath=Path("test.mp3")
    )  # Assume we have a real mp3 for testing
    with temp_mp3:
        assert isinstance(temp_mp3.mp3path, Path), "The path is not set correctly"
        assert temp_mp3.mp3path == Path(
            "test.mp3"
        ), "The path should be the same as the input file"



from pathlib import Path
from ak_transcribe.utils.config_parser import Config
from functools import lru_cache
from ak_transcribe.console import console
import subprocess
from typing import Literal

_config = Config()


class TempMp3:
    def __init__(self, filepath: Path):
        self.__ffmpeg_path: Path | str = Path(
            _config.get(keys=("ffmpeg", "cmd"), default="ffmpeg")
        )
        self.filepath = filepath
        assert self.filepath.exists()
        self.__unique_mp3file: bool = not self.is_audio(filepath=filepath)

    @lru_cache(maxsize=1)
    def __mp3_path(self) -> Path:
        if self.__unique_mp3file:
            return self.__convert_to_audio(file=self.filepath)
        else:
            return self.filepath

    def __enter__(self) -> Path:
        return self.__mp3_path()

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is not None:
            console.print(f"[red]An exception occurred:[/red] {exc_type} - {exc_value}")
        if not self.is_audio(filepath=self.filepath):
            self.__mp3_path().unlink()
        return False

    def __convert_to_audio(self, file: Path) -> Path:
        """Convert the given audio file to an ogg format."""
        outputpath = (
            Path(_config.get(keys=("ffmpeg", "transcode_dir"), default="."))
            / f"{file.stem}_temp.ogg"
        )
        # ffmpeg -i input.mp4 -ac 1 -ar 16000 -b:a 24k -c:a aac output.ogg
        command = f'"{self.__ffmpeg_path}" -loglevel quiet -stats -hide_banner -i "{file}" -vn -ac 1 -ar 16000 -c:a libopus -b:a 32k "{outputpath}" -y'
        with console.status("Converting to ogg...") as status:
            if run_command(command=command) != 0:
                raise Exception("Conversion to `.ogg` failed.")
        console.print(
            f"[green]Conversion successful![/green] Output saved to: {outputpath}"
        )
        return outputpath

    def is_video(self) -> bool:
        """Check if the given file is video"""
        return check_filemime(filepath=self.filepath, mime_str="video")

    @staticmethod
    def is_audio(filepath: Path) -> bool:
        """Check if the given file is audio"""
        return False  # Force Transcription
        return check_filemime(filepath=filepath, mime_str="audio")


def check_filemime(filepath: Path, mime_str: Literal["video", "audio"]):
    ext = filepath.suffix.casefold().strip()
    match mime_str.strip().casefold():
        case "video":
            _ext = (".mp4", ".mkv")
            return ext in _ext

        case "audio":
            _ext = (".mp3", ".m4a")
            return ext in _ext
        case _:
            raise Exception(f"Invalid mimetype: {mime_str}")


def run_command(command: str) -> int:
    """Run shell commands using Subprocess; Returns the process returncode"""
    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
    process.wait()
    return process.returncode



import json
import shutil

import warnings
from functools import lru_cache
from pathlib import Path

import requests
from groq import Groq
from ak_transcribe.filehandler import TempMp3, check_filemime, run_command

from ak_transcribe.utils.config_parser import Config
from ak_transcribe.console import console

_config = Config()


class TranscribedContent:
    def __init__(self, results: dict | Path) -> None:
        assert results is not None
        if isinstance(results, Path):
            assert results.suffix.casefold().endswith(".json")
            assert results.exists()
            with open(results, "r", encoding="utf-8") as f:
                self._results = json.load(f)
        elif isinstance(results, dict):
            self._results = results
        else:
            raise Exception(f"Expected type: dict|JSONfilepath, got {type(results)}")

    @property
    def txt(self) -> str:
        assert isinstance(self._results, dict), "Results not set yet"
        return "\n".join(
            [line["text"].strip() for line in self._results["segments"]]  # type: ignore
        )

    @property
    def srt(self) -> str:
        assert isinstance(self._results, dict), "Results not set yet"
        count = 0
        text_blob = ""
        for segment in self._results.get("segments", []):
            count += 1
            _timestamp_start: str = f"{count}\n{self.__srt_time(segment['start'])}"
            _timestamp_end: str = f"{self.__srt_time(segment['end'])}\n"
            _text: str = f"{segment['text'].replace('-->', '->').strip()}"
            text_blob += f"{_timestamp_start} --> {_timestamp_end}{_text}\n\n"
        return text_blob

    @property
    def json(self) -> str:
        """Return str representation of json object"""
        assert isinstance(self._results, dict), "Results not set yet"
        return json.dumps(self._results, indent=4)

    @staticmethod
    def __srt_time(_seconds: float) -> str:
        """Convert seconds into SRT (SubRip Text) time format."""
        assert _seconds >= 0, "non-negative timestamp expected"
        milliseconds: int = round(_seconds * 1000.0)
        hours: int = milliseconds // 3_600_000
        milliseconds -= hours * 3_600_000
        minutes: int = milliseconds // 60_000
        milliseconds -= minutes * 60_000
        seconds: int = milliseconds // 1_000
        milliseconds -= seconds * 1_000
        return (f"{hours}:") + f"{minutes:02d}:{seconds:02d},{milliseconds:03d}"

    def embed_srt(self, filepath: Path):
        if not check_filemime(filepath=filepath, mime_str="video"):
            console.print(f"[bold red] File ({filepath.name} is not a video file.)")
            return

        # Make sure the ".srt" file is setup
        temp_srt = filepath.with_suffix(".srt")
        if not temp_srt.is_file():
            with open(temp_srt, "w", encoding="utf-8") as f:
                f.write(self.srt)

        # Path definitions
        __ffmpeg_path: Path | str = Path(
            _config.get(keys=("ffmpeg", "cmd"), default="ffmpeg")
        )
        _tempoutput: Path = Path(".") / f"{filepath.stem}-Temp{filepath.suffix}"

        # FFmpeg command setup
        command = [
            f'"{__ffmpeg_path}"',
            "-loglevel quiet",
            "-stats -hide_banner",
            f'-i "{filepath}"',
            f'-i "{temp_srt}"',
            "-c:v copy",
            "-c:a copy",
            "-c:s mov_text",
            f'"{_tempoutput}"',
            "-y",
        ]
        command = " ".join(command)
        console.print(f"[green]FFmpeg cmd: [/green][grey]{command}[/grey]")
        # Run the process
        with console.status(f"Embedding subtitles into {filepath.name}..."):
            if run_command(command=command) != 0:
                raise Exception("Embedding subtitles failed.")
            shutil.move(_tempoutput, filepath)
            temp_srt.unlink()

        console.print(f"[green]Embedding Successful![/green]")


class OpenAI_CPU:
    def __init__(self) -> None:
        try:
            import whisper  # type: ignore
        except ModuleNotFoundError:
            raise Exception(
                (
                    "Whisper not installed. "
                    "Install using `pip install -r pyproject.toml --extra cpu`"
                )
            )

        model = _config.get(keys=("whisper", "model"), default="small.en")
        device = _config.get(keys=("whisper", "device"), default="cpu")
        self.__whisper = whisper.load_model(model, device=device)

    def process(self, filepath: Path) -> TranscribedContent:
        """Process the file and save results to `self._results`"""
        with TempMp3(filepath=filepath) as file:
            with warnings.catch_warnings(action="ignore"):
                console.print(f"[yellow]Processing file:[/yellow] {file}")
                results = self.__whisper.transcribe(str(file))
                console.print(f"[green]Processing complete![/green]")
                return TranscribedContent(results=results)


class OpenAI_webserver:
    def __init__(self) -> None:

        self.__base_url: str = _config.get(keys=("whisper", "url"), default="")
        assert self.__base_url != "", "Make sure [whisper][url] is set in `config.toml`"
        requests.get(self.__base_url).raise_for_status()

        self.__transcript_url = f"{self.__base_url.rstrip('/')}/asr?encode=true&task=transcribe&word_timestamps=true&output=json"
        console.print(f"[green]Using Whisper webserver")

    def process(self, filepath: Path) -> TranscribedContent:
        """Process the file and save results to `self._results`"""
        json_path = filepath.with_suffix(".json")
        if json_path.is_file():
            console.print(f"[green]Existing Transcription Found![/green]")
            with open(json_path, "r", encoding="utf-8") as f:
                return TranscribedContent(results=json.load(f))

        with TempMp3(filepath=filepath) as file:
            audio_file_path = str(file)
            with open(audio_file_path, "rb") as f:
                audio_data = f.read()
            _files = {
                "audio_file": (
                    audio_file_path,
                    audio_data,
                    "audio/mpeg",
                )
            }
            with console.status("Sending request to server...") as status:
                response = requests.post(self.__transcript_url, files=_files)
                response.raise_for_status()
                console.print(f"[green]Received transcription successfully![/green]")
                return TranscribedContent(results=response.json())


class GroqTranscriber:
    def __init__(self):
        self.__client = Groq(api_key=_config.get(keys=("groq", "key"), default=""))

    def process(self, filepath: Path) -> TranscribedContent:

        with TempMp3(filepath=filepath) as file:
            with open(str(file), "rb") as f:
                with console.status("Sending request to server...") as status:

                    transcription = self.__client.audio.transcriptions.create(
                        file=(filepath.name, f.read()),
                        model="distil-whisper-large-v3-en",
                        response_format="verbose_json",
                    )
                    console.print(
                        f"[green]Received transcription successfully![/green]"
                    )
                return TranscribedContent(json.loads(transcription.model_dump_json()))


@lru_cache(maxsize=1)
def _get_processor() -> OpenAI_webserver | OpenAI_CPU:
    if _config.get(keys=("groq", "key"), default="") != "":
        print("Using Groq Transcriber")
        return GroqTranscriber()
    try:
        print("Using OpenAI Webserver")
        return OpenAI_webserver()
    except Exception as e:
        console.print(f"[red]Web server processor failed:[/red] {e}")
        console.print("[yellow]Attempting CPU Processing...[/yellow]")
        return OpenAI_CPU()


Transcriber: OpenAI_webserver | OpenAI_CPU | GroqTranscriber = _get_processor()
